#include <iostream>
#include <iomanip>
#include <cassert>
#include <cstdint>

using namespace std;

// Функция для преобразования значения nibble (0-15) в шестнадцатеричный символ
char nibble_to_hex(uint8_t i) {
    assert(0x0 <= i && i <= 0xf);
    char digits[] = "0123456789abcdef";
    return digits[i];
}

// Функция для печати одного байта в шестнадцатеричном представлении
void print_in_hex(uint8_t byte) {
    cout << nibble_to_hex(byte >> 4) << nibble_to_hex(byte & 0xf);
}

// Функция для преобразования указателя void* в указатель на байты uint8_t*
const uint8_t *as_bytes(const void *data) {
    return reinterpret_cast<const uint8_t *>(data);
}

// Функция для печати блока данных в шестнадцатеричном представлении
void print_in_hex(const void *data, size_t size) {
    const uint8_t *bytes = as_bytes(data);
    for (size_t i = 0; i < size; i++) {
        print_in_hex(bytes[i]);
        if ((i + 1) % 16 == 0) {
            cout << '\n';
        } else {
            cout << ' ';
        }
    }
}

// Функция для получения бита с номером bit из байта byte
char bit_digit(uint8_t byte, uint8_t bit) {
    if (byte & (0x1 << bit)) {
        return '1';
    }
    return '0';
}

// Функция для печати одного байта в двоичном представлении
void print_in_binary(uint8_t byte) {
    for (uint8_t bit = 7; bit < 8; bit--) {
        cout << bit_digit(byte, bit);
    }
}

// Функция для печати блока данных в двоичном представлении
void print_in_binary(const void *data, size_t size) {
    const uint8_t *bytes = as_bytes(data);
    for (size_t i = 0; i < size; i++) {
        print_in_binary(bytes[i]);
        if ((i + 1) % 4 == 0) {
            cout << '\n';
        } else {
            cout << ' ';
        }
    }
}

int main() {
    uint32_t u32 = 0x42;
    cout << "u32 bytes: (hex)\n";
    print_in_hex(&u32, sizeof(u32));
    cout << "\nu32 bytes: (binary)\n";
    print_in_binary(&u32, sizeof(u32));
    return 0;
}